@@ -1,11 +1,11 @@
-from graphene_django import DjangoObjectType
+import graphene_django_optimizer as gql_optimizer
 
 from nautobot.circuits.models import CircuitTermination
 from nautobot.circuits.filters import CircuitTerminationFilterSet
 from nautobot.dcim.graphql.mixins import PathEndpointMixin
 
 
-class CircuitTerminationType(DjangoObjectType, PathEndpointMixin):
+class CircuitTerminationType(gql_optimizer.OptimizedDjangoObjectType, PathEndpointMixin):
     """Graphql Type Object for CircuitTermination model."""
 
     class Meta:
