@@ -8,6 +8,7 @@ from django.test import TestCase, override_settings
 from django.test.client import RequestFactory
 from django.urls import reverse
 from graphql import GraphQLError
+import graphene.types
 from graphene_django import DjangoObjectType
 from graphene_django.settings import graphene_settings
 from graphql.error.located_error import GraphQLLocatedError
@@ -30,10 +31,22 @@ from nautobot.core.graphql.schema import (
     extend_schema_type_relationships,
     extend_schema_type_null_field_choice,
 )
-from nautobot.dcim.choices import InterfaceTypeChoices, InterfaceModeChoices
+from nautobot.dcim.choices import InterfaceTypeChoices, InterfaceModeChoices, PortTypeChoices
 from nautobot.dcim.filters import DeviceFilterSet, SiteFilterSet
 from nautobot.dcim.graphql.types import DeviceType as DeviceTypeGraphQL
-from nautobot.dcim.models import Cable, Device, DeviceRole, DeviceType, Interface, Manufacturer, Rack, Region, Site
+from nautobot.dcim.models import (
+    Cable,
+    Device,
+    DeviceRole,
+    DeviceType,
+    FrontPort,
+    Interface,
+    Manufacturer,
+    Rack,
+    RearPort,
+    Region,
+    Site,
+)
 from nautobot.extras.choices import CustomFieldTypeChoices
 from nautobot.utilities.testing.utils import create_test_user
 
@@ -194,13 +207,14 @@ class GraphQLExtendSchemaType(TestCase):
         self.assertTrue(hasattr(schema, "resolve_tags"))
         self.assertIsInstance(getattr(schema, "resolve_tags"), types.FunctionType)
 
-    def test_extend_custom_context(self):
+    def test_extend_config_context(self):
 
         schema = extend_schema_type_config_context(DeviceTypeGraphQL, Device)
         self.assertIn("config_context", schema._meta.fields.keys())
 
     def test_extend_schema_device(self):
 
+        # The below *will* log an error as DeviceTypeGraphQL has already been extended automatically...?
         schema = extend_schema_type(DeviceTypeGraphQL)
         self.assertIn("config_context", schema._meta.fields.keys())
         self.assertTrue(hasattr(schema, "resolve_tags"))
@@ -228,7 +242,7 @@ class GraphQLExtendSchemaRelationship(TestCase):
         rack_ct = ContentType.objects.get_for_model(Rack)
         vlan_ct = ContentType.objects.get_for_model(VLAN)
 
-        self.m2m_1 = Relationship.objects.create(
+        self.m2m_1 = Relationship(
             name="Vlan to Rack",
             slug="vlan-rack",
             source_type=rack_ct,
@@ -237,24 +251,27 @@ class GraphQLExtendSchemaRelationship(TestCase):
             destination_label="My Racks",
             type="many-to-many",
         )
+        self.m2m_1.validated_save()
 
-        self.m2m_2 = Relationship.objects.create(
+        self.m2m_2 = Relationship(
             name="Another Vlan to Rack",
             slug="vlan-rack-2",
             source_type=rack_ct,
             destination_type=vlan_ct,
             type="many-to-many",
         )
+        self.m2m_2.validated_save()
 
-        self.o2m_1 = Relationship.objects.create(
+        self.o2m_1 = Relationship(
             name="generic site to vlan",
             slug="site-vlan",
             source_type=site_ct,
             destination_type=vlan_ct,
             type="one-to-many",
         )
+        self.o2m_1.validated_save()
 
-        self.o2o_1 = Relationship.objects.create(
+        self.o2o_1 = Relationship(
             name="Primary Rack per Site",
             slug="primary-rack-site",
             source_type=rack_ct,
@@ -263,40 +280,86 @@ class GraphQLExtendSchemaRelationship(TestCase):
             destination_label="Primary Rack",
             type="one-to-one",
         )
+        self.o2o_1.validated_save()
 
-        self.sites = [
-            Site.objects.create(name="Site A", slug="site-a"),
-            Site.objects.create(name="Site B", slug="site-b"),
-            Site.objects.create(name="Site C", slug="site-c"),
-        ]
+        self.o2os_1 = Relationship(
+            name="Redundant Site",
+            slug="redundant-site",
+            source_type=site_ct,
+            destination_type=site_ct,
+            type="symmetric-one-to-one",
+        )
+        self.o2os_1.validated_save()
 
-        self.racks = [
-            Rack.objects.create(name="Rack A", site=self.sites[0]),
-            Rack.objects.create(name="Rack B", site=self.sites[1]),
-            Rack.objects.create(name="Rack C", site=self.sites[2]),
-        ]
+        self.o2m_same_type_1 = Relationship(
+            name="Some sort of site hierarchy?",
+            slug="site-hierarchy",
+            source_type=site_ct,
+            destination_type=site_ct,
+            type="one-to-many",
+        )
+        self.o2m_same_type_1.validated_save()
 
-        self.vlans = [
-            VLAN.objects.create(name="VLAN A", vid=100, site=self.sites[0]),
-            VLAN.objects.create(name="VLAN B", vid=100, site=self.sites[1]),
-            VLAN.objects.create(name="VLAN C", vid=100, site=self.sites[2]),
-        ]
+        self.site_schema = generate_schema_type(app_name="dcim", model=Site)
+        self.vlan_schema = generate_schema_type(app_name="ipam", model=VLAN)
 
-        self.schema = generate_schema_type(app_name="dcim", model=Site)
+    def test_extend_relationship_default_prefix(self):
+        """Verify that relationships are correctly added to the schema."""
+        schema = extend_schema_type_relationships(self.vlan_schema, VLAN)
 
-    @override_settings(GRAPHQL_CUSTOM_RELATIONSHIP_PREFIX="pr")
-    def test_extend_relationship_w_prefix(self):
-
-        schema = extend_schema_type_relationships(self.schema, Site)
-
-        datas = [
-            {"field_slug": "primary-rack-site"},
-            {"field_slug": "site-vlan"},
-        ]
-
-        for data in datas:
-            field_name = f"rel_{str_to_var_name(data['field_slug'])}"
+        # Relationships on VLAN
+        for rel, peer_side in [
+            (self.m2m_1, "source"),
+            (self.m2m_2, "source"),
+            (self.o2m_1, "source"),
+        ]:
+            field_name = f"rel_{str_to_var_name(rel.slug)}"
             self.assertIn(field_name, schema._meta.fields.keys())
+            self.assertIsInstance(schema._meta.fields[field_name], graphene.types.field.Field)
+            if rel.has_many(peer_side):
+                self.assertIsInstance(schema._meta.fields[field_name].type, graphene.types.structures.List)
+            else:
+                self.assertNotIsInstance(schema._meta.fields[field_name].type, graphene.types.structures.List)
+
+        # Relationships not on VLAN
+        for rel in [self.o2o_1, self.o2os_1]:
+            field_name = f"rel_{str_to_var_name(rel.slug)}"
+            self.assertNotIn(field_name, schema._meta.fields.keys())
+
+    @override_settings(GRAPHQL_RELATIONSHIP_PREFIX="pr")
+    def test_extend_relationship_w_prefix(self):
+        """Verify that relationships are correctly added to the schema when using a custom prefix setting."""
+        schema = extend_schema_type_relationships(self.site_schema, Site)
+
+        # Relationships on Site
+        for rel, peer_side in [
+            (self.o2m_1, "destination"),
+            (self.o2o_1, "source"),
+            (self.o2os_1, "peer"),
+        ]:
+            field_name = f"pr_{str_to_var_name(rel.slug)}"
+            self.assertIn(field_name, schema._meta.fields.keys())
+            self.assertIsInstance(schema._meta.fields[field_name], graphene.types.field.Field)
+            if rel.has_many(peer_side):
+                self.assertIsInstance(schema._meta.fields[field_name].type, graphene.types.structures.List)
+            else:
+                self.assertNotIsInstance(schema._meta.fields[field_name].type, graphene.types.structures.List)
+
+        # Special handling of same-type non-symmetric relationships
+        for rel in [self.o2m_same_type_1]:
+            for peer_side in ["source", "destination"]:
+                field_name = f"pr_{str_to_var_name(rel.slug)}_{peer_side}"
+                self.assertIn(field_name, schema._meta.fields.keys())
+                self.assertIsInstance(schema._meta.fields[field_name], graphene.types.field.Field)
+                if rel.has_many(peer_side):
+                    self.assertIsInstance(schema._meta.fields[field_name].type, graphene.types.structures.List)
+                else:
+                    self.assertNotIsInstance(schema._meta.fields[field_name].type, graphene.types.structures.List)
+
+        # Relationships not on Site
+        for rel in [self.m2m_1, self.m2m_2]:
+            field_name = f"pr_{str_to_var_name(rel.slug)}"
+            self.assertNotIn(field_name, schema._meta.fields.keys())
 
 
 class GraphQLSearchParameters(TestCase):
@@ -425,6 +488,14 @@ class GraphQLAPIPermissionTest(TestCase):
         }
         """
 
+        self.get_rack_query = """
+        query ($id: ID!) {
+            rack (id: $id) {
+                name
+            }
+        }
+        """
+
     def test_graphql_api_token_with_perm(self):
         """Validate that users can query based on their permissions."""
         # First user
@@ -516,6 +587,14 @@ class GraphQLAPIPermissionTest(TestCase):
         names = [item["name"] for item in response.data["data"]["racks"]]
         self.assertEqual(names, ["Rack 2-1", "Rack 2-2"])
 
+    def test_graphql_single_object_query(self):
+        """Validate graphql query for a single object as opposed to a set of objects also works."""
+        payload = {"query": self.get_rack_query, "variables": {"id": Rack.objects.first().pk}}
+        response = self.clients[2].post(self.api_url, payload, format="json")
+        self.assertEqual(response.status_code, status.HTTP_200_OK)
+        self.assertIsInstance(response.data["data"]["rack"], dict)
+        self.assertEqual(response.data["data"]["rack"]["name"], Rack.objects.first().name)
+
     def test_graphql_query_multi_level(self):
         """Validate request with multiple levels return the proper information, following the permissions."""
         response = self.clients[0].post(self.api_url, {"query": self.get_sites_racks_query}, format="json")
@@ -592,6 +671,40 @@ class GraphQLQueryTest(TestCase):
             comments="First Device",
         )
 
+        self.device1_rear_ports = (
+            RearPort.objects.create(device=self.device1, name="Rear Port 1", type=PortTypeChoices.TYPE_8P8C),
+            RearPort.objects.create(device=self.device1, name="Rear Port 2", type=PortTypeChoices.TYPE_8P8C),
+            RearPort.objects.create(device=self.device1, name="Rear Port 3", type=PortTypeChoices.TYPE_8P8C),
+            RearPort.objects.create(device=self.device1, name="Rear Port 4", type=PortTypeChoices.TYPE_8P8C),
+        )
+
+        self.device1_frontports = [
+            FrontPort.objects.create(
+                device=self.device1,
+                name="Front Port 1",
+                type=PortTypeChoices.TYPE_8P8C,
+                rear_port=self.device1_rear_ports[0],
+            ),
+            FrontPort.objects.create(
+                device=self.device1,
+                name="Front Port 2",
+                type=PortTypeChoices.TYPE_8P8C,
+                rear_port=self.device1_rear_ports[1],
+            ),
+            FrontPort.objects.create(
+                device=self.device1,
+                name="Front Port 3",
+                type=PortTypeChoices.TYPE_8P8C,
+                rear_port=self.device1_rear_ports[2],
+            ),
+            FrontPort.objects.create(
+                device=self.device1,
+                name="Front Port 4",
+                type=PortTypeChoices.TYPE_8P8C,
+                rear_port=self.device1_rear_ports[3],
+            ),
+        ]
+
         self.interface11 = Interface.objects.create(
             name="Int1",
             type=InterfaceTypeChoices.TYPE_VIRTUAL,
@@ -699,28 +812,45 @@ class GraphQLQueryTest(TestCase):
             return document.execute(context_value=self.request)
 
     @override_settings(EXEMPT_VIEW_PERMISSIONS=["*"])
-    def test_query_config_context(self):
+    def test_query_config_context_and_custom_field_data(self):
 
         query = """
         query {
             devices {
                 name
                 config_context
+                _custom_field_data
+            }
+            device (id: "%s") {
+                name
+                config_context
+                _custom_field_data
             }
         }
-        """
+        """ % (
+            self.device1.id,
+        )
 
         expected_data = {"a": 123, "b": 456, "c": 777}
 
         result = self.execute_query(query)
 
         self.assertIsInstance(result.data["devices"], list)
+        self.assertIsInstance(result.data["device"], dict)
+
         device_names = [item["name"] for item in result.data["devices"]]
         self.assertEqual(sorted(device_names), ["Device 1", "Device 2", "Device 3"])
+        self.assertEqual(result.data["device"]["name"], "Device 1")
 
-        config_context = [item["config_context"] for item in result.data["devices"]]
-        self.assertIsInstance(config_context[0], dict)
-        self.assertDictEqual(config_context[0], expected_data)
+        config_contexts = [item["config_context"] for item in result.data["devices"]]
+        self.assertIsInstance(config_contexts[0], dict)
+        self.assertDictEqual(config_contexts[0], expected_data)
+        self.assertEqual(result.data["device"]["config_context"], expected_data)
+
+        custom_field_data = [item["_custom_field_data"] for item in result.data["devices"]]
+        self.assertIsInstance(custom_field_data[0], dict)
+        self.assertEqual(custom_field_data[0], {})
+        self.assertEqual(result.data["device"]["_custom_field_data"], {})
 
     @override_settings(EXEMPT_VIEW_PERMISSIONS=["*"])
     def test_query_device_role_filter(self):
@@ -876,6 +1006,7 @@ query {
                 virtual_machine { name }
             }
         }
+        family
         interface { name }
         vminterface { name }
     }
@@ -888,6 +1019,7 @@ query {
                 entry["address"], (str(self.ipaddr1.address), str(self.ipaddr2.address), str(self.vmipaddr.address))
             )
             self.assertIn("assigned_object", entry)
+            self.assertIn(entry["family"], (4, 6))
             if entry["address"] == str(self.vmipaddr.address):
                 self.assertEqual(entry["assigned_object"]["name"], self.vminterface.name)
                 self.assertEqual(entry["vminterface"]["name"], self.vminterface.name)
@@ -928,6 +1060,40 @@ query {
                 self.assertIsNone(result.errors)
                 self.assertEqual(len(result.data["cables"]), nbr_expected_results)
 
+    @override_settings(EXEMPT_VIEW_PERMISSIONS=["*"])
+    def test_query_frontport_filter_second_level(self):
+        """Test "second-level" filtering of FrontPorts within a Devices query."""
+
+        filters = (
+            (f'name: "{self.device1_frontports[0].name}"', 1),
+            (f'device: "{self.device1.name}"', 4),
+            (f'_type: "{PortTypeChoices.TYPE_8P8C}"', 4),
+        )
+
+        for filter, nbr_expected_results in filters:
+            with self.subTest(msg=f"Checking {filter}", filter=filter, nbr_expected_results=nbr_expected_results):
+                query = "query { devices{ frontports(" + filter + "){ id }}}"
+                result = self.execute_query(query)
+                self.assertIsNone(result.errors)
+                self.assertEqual(len(result.data["devices"][0]["frontports"]), nbr_expected_results)
+
+    @override_settings(EXEMPT_VIEW_PERMISSIONS=["*"])
+    def test_query_frontport_filter_third_level(self):
+        """Test "third-level" filtering of FrontPorts within Devices within Sites."""
+
+        filters = (
+            (f'name: "{self.device1_frontports[0].name}"', 1),
+            (f'device: "{self.device1.name}"', 4),
+            (f'_type: "{PortTypeChoices.TYPE_8P8C}"', 4),
+        )
+
+        for filter, nbr_expected_results in filters:
+            with self.subTest(msg=f"Checking {filter}", filter=filter, nbr_expected_results=nbr_expected_results):
+                query = "query { sites{ devices{ frontports(" + filter + "){ id }}}}"
+                result = self.execute_query(query)
+                self.assertIsNone(result.errors)
+                self.assertEqual(len(result.data["sites"][0]["devices"][0]["frontports"]), nbr_expected_results)
+
     @override_settings(EXEMPT_VIEW_PERMISSIONS=["*"])
     def test_query_interfaces_filter(self):
         """Test custom interface filter fields and boolean, not other concrete fields."""
@@ -951,6 +1117,44 @@ query {
                 self.assertIsNone(result.errors)
                 self.assertEqual(len(result.data["interfaces"]), nbr_expected_results)
 
+    @override_settings(EXEMPT_VIEW_PERMISSIONS=["*"])
+    def test_query_interfaces_filter_second_level(self):
+        """Test "second-level" filtering of Interfaces within a Devices query."""
+
+        filters = (
+            (f'device_id: "{self.device1.id}"', 2),
+            ('kind: "virtual"', 2),
+            ('mac_address: "00:11:11:11:11:11"', 1),
+            ("vlan: 100", 1),
+            (f'vlan_id: "{self.vlan1.id}"', 1),
+        )
+
+        for filter, nbr_expected_results in filters:
+            with self.subTest(msg=f"Checking {filter}", filter=filter, nbr_expected_results=nbr_expected_results):
+                query = "query { devices{ interfaces(" + filter + "){ id }}}"
+                result = self.execute_query(query)
+                self.assertIsNone(result.errors)
+                self.assertEqual(len(result.data["devices"][0]["interfaces"]), nbr_expected_results)
+
+    @override_settings(EXEMPT_VIEW_PERMISSIONS=["*"])
+    def test_query_interfaces_filter_third_level(self):
+        """Test "third-level" filtering of Interfaces within Devices within Sites."""
+
+        filters = (
+            (f'device_id: "{self.device1.id}"', 2),
+            ('kind: "virtual"', 2),
+            ('mac_address: "00:11:11:11:11:11"', 1),
+            ("vlan: 100", 1),
+            (f'vlan_id: "{self.vlan1.id}"', 1),
+        )
+
+        for filter, nbr_expected_results in filters:
+            with self.subTest(msg=f"Checking {filter}", filter=filter, nbr_expected_results=nbr_expected_results):
+                query = "query { sites{ devices{ interfaces(" + filter + "){ id }}}}"
+                result = self.execute_query(query)
+                self.assertIsNone(result.errors)
+                self.assertEqual(len(result.data["sites"][0]["devices"][0]["interfaces"]), nbr_expected_results)
+
     @override_settings(EXEMPT_VIEW_PERMISSIONS=["*"])
     def test_query_interfaces_connected_endpoint(self):
         """Test querying interfaces for their connected endpoints."""
