@@ -1,9 +1,18 @@
 from django.contrib.auth import get_user_model
+from django.test import override_settings
 from django.urls import reverse
 from rest_framework import status
 
-from nautobot.dcim.choices import *
-from nautobot.dcim.constants import *
+from constance.test import override_config
+
+from nautobot.dcim.choices import (
+    InterfaceModeChoices,
+    InterfaceTypeChoices,
+    PortTypeChoices,
+    PowerFeedTypeChoices,
+    SubdeviceRoleChoices,
+)
+
 from nautobot.dcim.models import (
     Cable,
     ConsolePort,
@@ -38,7 +47,7 @@ from nautobot.dcim.models import (
     Site,
     VirtualChassis,
 )
-from nautobot.extras.models import ConfigContextSchema, Status
+from nautobot.extras.models import ConfigContextSchema, SecretsGroup, Status
 from nautobot.ipam.models import VLAN
 from nautobot.utilities.testing import APITestCase, APIViewTestCases
 from nautobot.virtualization.models import Cluster, ClusterType
@@ -106,10 +115,12 @@ class RegionTest(APIViewTestCases.APIViewTestCase):
             "name": "Region 6",
             "slug": "region-6",
         },
+        {"name": "Region 7"},
     ]
     bulk_update_data = {
         "description": "New description",
     }
+    slug_source = "name"
 
     @classmethod
     def setUpTestData(cls):
@@ -126,6 +137,7 @@ class SiteTest(APIViewTestCases.APIViewTestCase):
         "status": "planned",
     }
     choices_fields = ["status"]
+    slug_source = "name"
 
     @classmethod
     def setUpTestData(cls):
@@ -166,6 +178,7 @@ class SiteTest(APIViewTestCases.APIViewTestCase):
                 "region": regions[1].pk,
                 "status": "active",
             },
+            {"name": "Site 7", "region": regions[1].pk, "status": "active"},
         ]
 
     def test_time_zone_field_post_null(self):
@@ -254,6 +267,7 @@ class RackGroupTest(APIViewTestCases.APIViewTestCase):
     bulk_update_data = {
         "description": "New description",
     }
+    slug_source = "name"
 
     @classmethod
     def setUpTestData(cls):
@@ -306,6 +320,11 @@ class RackGroupTest(APIViewTestCases.APIViewTestCase):
                 "site": sites[1].pk,
                 "parent": parent_rack_groups[1].pk,
             },
+            {
+                "name": "Test Rack Group 7",
+                "site": sites[1].pk,
+                "parent": parent_rack_groups[1].pk,
+            },
         ]
 
 
@@ -328,10 +347,15 @@ class RackRoleTest(APIViewTestCases.APIViewTestCase):
             "slug": "rack-role-6",
             "color": "ffff00",
         },
+        {
+            "name": "Rack Role 7",
+            "color": "ffff00",
+        },
     ]
     bulk_update_data = {
         "description": "New description",
     }
+    slug_source = "name"
 
     @classmethod
     def setUpTestData(cls):
@@ -467,6 +491,36 @@ class RackTest(APIViewTestCases.APIViewTestCase):
         response = self.client.get(url, **self.header)
         self.assertHttpStatus(response, status.HTTP_200_OK)
         self.assertEqual(response.get("Content-Type"), "image/svg+xml")
+        self.assertIn(b'class="slot" height="22" width="230"', response.content)
+
+    @override_settings(RACK_ELEVATION_DEFAULT_UNIT_HEIGHT=27, RACK_ELEVATION_DEFAULT_UNIT_WIDTH=255)
+    @override_config(RACK_ELEVATION_DEFAULT_UNIT_HEIGHT=19, RACK_ELEVATION_DEFAULT_UNIT_WIDTH=190)
+    def test_get_rack_elevation_svg_settings_overridden(self):
+        """
+        GET a single rack elevation in SVG format, with Django settings specifying a non-standard unit size.
+        """
+        rack = Rack.objects.first()
+        self.add_permissions("dcim.view_rack")
+        url = "{}?render=svg".format(reverse("dcim-api:rack-elevation", kwargs={"pk": rack.pk}))
+
+        response = self.client.get(url, **self.header)
+        self.assertHttpStatus(response, status.HTTP_200_OK)
+        self.assertEqual(response.get("Content-Type"), "image/svg+xml")
+        self.assertIn(b'class="slot" height="27" width="255"', response.content)
+
+    @override_config(RACK_ELEVATION_DEFAULT_UNIT_HEIGHT=19, RACK_ELEVATION_DEFAULT_UNIT_WIDTH=190)
+    def test_get_rack_elevation_svg_config_overridden(self):
+        """
+        GET a single rack elevation in SVG format, with Constance config specifying a non-standard unit size.
+        """
+        rack = Rack.objects.first()
+        self.add_permissions("dcim.view_rack")
+        url = "{}?render=svg".format(reverse("dcim-api:rack-elevation", kwargs={"pk": rack.pk}))
+
+        response = self.client.get(url, **self.header)
+        self.assertHttpStatus(response, status.HTTP_200_OK)
+        self.assertEqual(response.get("Content-Type"), "image/svg+xml")
+        self.assertIn(b'class="slot" height="19" width="190"', response.content)
 
 
 class RackReservationTest(APIViewTestCases.APIViewTestCase):
@@ -532,10 +586,14 @@ class ManufacturerTest(APIViewTestCases.APIViewTestCase):
             "name": "Manufacturer 6",
             "slug": "manufacturer-6",
         },
+        {
+            "name": "Manufacturer 7",
+        },
     ]
     bulk_update_data = {
         "description": "New description",
     }
+    slug_source = "name"
 
     @classmethod
     def setUpTestData(cls):
@@ -560,6 +618,7 @@ class DeviceTypeTest(APIViewTestCases.APIViewTestCase):
         "part_number": "ABC123",
     }
     choices_fields = ["subdevice_role"]
+    slug_source = "model"
 
     @classmethod
     def setUpTestData(cls):
@@ -589,6 +648,10 @@ class DeviceTypeTest(APIViewTestCases.APIViewTestCase):
                 "model": "Device Type 6",
                 "slug": "device-type-6",
             },
+            {
+                "manufacturer": manufacturers[1].pk,
+                "model": "Device Type 7",
+            },
         ]
 
 
@@ -954,10 +1017,15 @@ class DeviceRoleTest(APIViewTestCases.APIViewTestCase):
             "slug": "device-role-6",
             "color": "ffff00",
         },
+        {
+            "name": "Device Role 7",
+            "color": "ffff00",
+        },
     ]
     bulk_update_data = {
         "description": "New description",
     }
+    slug_source = "name"
 
     @classmethod
     def setUpTestData(cls):
@@ -983,10 +1051,14 @@ class PlatformTest(APIViewTestCases.APIViewTestCase):
             "name": "Platform 6",
             "slug": "platform-6",
         },
+        {
+            "name": "Platform 7",
+        },
     ]
     bulk_update_data = {
         "description": "New description",
     }
+    slug_source = "name"
 
     @classmethod
     def setUpTestData(cls):
@@ -1038,6 +1110,11 @@ class DeviceTest(APIViewTestCases.APIViewTestCase):
             Cluster.objects.create(name="Cluster 2", type=cluster_type),
         )
 
+        secrets_groups = (
+            SecretsGroup.objects.create(name="Secrets Group 1", slug="secrets-group-1"),
+            SecretsGroup.objects.create(name="Secrets Group 2", slug="secrets-group-2"),
+        )
+
         Device.objects.create(
             device_type=device_types[0],
             device_role=device_roles[0],
@@ -1046,6 +1123,7 @@ class DeviceTest(APIViewTestCases.APIViewTestCase):
             site=sites[0],
             rack=racks[0],
             cluster=clusters[0],
+            secrets_group=secrets_groups[0],
             local_context_data={"A": 1},
         ),
         Device.objects.create(
@@ -1056,6 +1134,7 @@ class DeviceTest(APIViewTestCases.APIViewTestCase):
             site=sites[0],
             rack=racks[0],
             cluster=clusters[0],
+            secrets_group=secrets_groups[0],
             local_context_data={"B": 2},
         ),
         Device.objects.create(
@@ -1066,6 +1145,7 @@ class DeviceTest(APIViewTestCases.APIViewTestCase):
             site=sites[0],
             rack=racks[0],
             cluster=clusters[0],
+            secrets_group=secrets_groups[0],
             local_context_data={"C": 3},
         ),
 
@@ -1086,6 +1166,7 @@ class DeviceTest(APIViewTestCases.APIViewTestCase):
                 "site": sites[1].pk,
                 "rack": racks[1].pk,
                 "cluster": clusters[1].pk,
+                "secrets_group": secrets_groups[1].pk,
             },
             {
                 "device_type": device_types[1].pk,
@@ -1095,6 +1176,7 @@ class DeviceTest(APIViewTestCases.APIViewTestCase):
                 "site": sites[1].pk,
                 "rack": racks[1].pk,
                 "cluster": clusters[1].pk,
+                "secrets_group": secrets_groups[1].pk,
             },
             {
                 "device_type": device_types[1].pk,
@@ -1104,6 +1186,7 @@ class DeviceTest(APIViewTestCases.APIViewTestCase):
                 "site": sites[1].pk,
                 "rack": racks[1].pk,
                 "cluster": clusters[1].pk,
+                "secrets_group": secrets_groups[1].pk,
             },
         ]
 
