@@ -5,6 +5,7 @@ import json
 import logging
 import os
 import pkgutil
+import sys
 import shutil
 import traceback
 import warnings
@@ -14,11 +15,12 @@ from cacheops import cached
 from db_file_storage.form_widgets import DBClearableFileInput
 from django import forms
 from django.conf import settings
+from django.contrib.auth import get_user_model
+from django.contrib.contenttypes.models import ContentType
 from django.core.files.uploadedfile import InMemoryUploadedFile
 from django.core.validators import RegexValidator
 from django.db import transaction
 from django.db.models import Model
-from django.core.exceptions import ObjectDoesNotExist
 from django.db.models.query import QuerySet
 from django.forms import ValidationError
 from django.utils import timezone
@@ -31,7 +33,7 @@ from .choices import JobResultStatusChoices, LogLevelChoices
 from .context_managers import change_logging
 from .datasources.git import ensure_git_repository
 from .forms import JobForm
-from .models import FileProxy, GitRepository
+from .models import FileProxy, GitRepository, ScheduledJob
 from .registry import registry
 
 from nautobot.core.celery import nautobot_task
@@ -48,6 +50,9 @@ from nautobot.utilities.forms import (
 )
 
 
+User = get_user_model()
+
+
 __all__ = [
     "Job",
     "BooleanVar",
@@ -89,27 +94,16 @@ class BaseJob:
         - commit_default (bool)
         - field_order (list)
         - read_only (bool)
+        - approval_required (bool)
         """
 
         pass
 
-    @staticmethod
-    def _results_struct():
-        return OrderedDict(
-            [
-                ("success", 0),
-                ("info", 0),
-                ("warning", 0),
-                ("failure", 0),
-                ("log", []),
-            ]
-        )
-
     def __init__(self):
         self.logger = logging.getLogger(f"nautobot.jobs.{self.__class__.__name__}")
 
         self.request = None
-        self.active_test = None
+        self.active_test = "main"
         self.failed = False
         self._job_result = None
 
@@ -187,6 +181,10 @@ class BaseJob:
     def read_only(cls):
         return getattr(cls.Meta, "read_only", False)
 
+    @classproperty
+    def approval_required(cls):
+        return getattr(cls.Meta, "approval_required", False)
+
     @classmethod
     def _get_vars(cls):
         vars = OrderedDict()
@@ -215,71 +213,31 @@ class BaseJob:
     def job_result(self, value):
         # Initialize job_result data format for our usage
         value.data = OrderedDict()
-        value.data["total"] = self._results_struct()
-        del value.data["total"]["log"]
-        for method_name in self.test_methods:
-            value.data[method_name] = self._results_struct()
-        # Only initialize results for run and post_run if they're actually implemented
-        if self.run.__func__ != BaseJob.run:
-            value.data["run"] = self._results_struct()
-        if self.post_run.__func__ != BaseJob.post_run:
-            value.data["post_run"] = self._results_struct()
 
         self._job_result = value
 
     @property
     def results(self):
         """
-        The results (log messages and final output) generated by this job.
+        The results generated by this job.
+        ** If you need the logs, you will need to filter on JobLogEntry **
+            Ex.
+            from nautobot.extras.models import JogLogEntry
+
+            JobLogEntry.objects.filter(job_result=self.job_result, <other criteria>)
 
         {
-            "total": {
-                "success": 0,
-                "info": 3,
-                "warning": 6,
-                "failure": 9,
-            }
-            "run": {
-                "success": 0,
-                "info": 1,
-                "warning": 2,
-                "failure": 3,
-                "log": [
-                    (timestamp, level, object_name, object_url, message),
-                    (timestamp, level, object_name, object_url, message),
-                    ...
-                ],
-            },
-            "test_function": {
-                "success": 0,
-                "info": 1,
-                "warning": 2,
-                "failure": 3,
-                "log": [
-                    (timestamp, level, object_name, object_url, message),
-                    (timestamp, level, object_name, object_url, message),
-                    ...
-                ],
-            },
-            "post_run": {
-                "success": 0,
-                "info": 1,
-                "warning": 2,
-                "failure": 3,
-                "log": [
-                    (timestamp, level, object_name, object_url, message),
-                    (timestamp, level, object_name, object_url, message),
-                    ...
-                ],
-            },
             "output": "...",
         }
         """
         return self.job_result.data if self.job_result else None
 
-    def as_form(self, data=None, files=None, initial=None):
+    def as_form(self, data=None, files=None, initial=None, approval_view=False):
         """
         Return a Django form suitable for populating the context data required to run this Job.
+
+        `approval_view` will disable all fields from modification and is used to display the form
+        during a approval review workflow.
         """
         fields = {name: var.as_field() for name, var in self._get_vars().items()}
         FormClass = type("JobForm", (JobForm,), fields)
@@ -299,6 +257,14 @@ class BaseJob:
         if field_order:
             form.order_fields(field_order)
 
+        if approval_view:
+            # Set `disabled=True` on all fields
+            for _, field in form.fields.items():
+                field.disabled = True
+
+            # Alter the commit help text to avoid confusion concerning approval dry-runs
+            form.fields["_commit"].help_text = "Commit changes to the database"
+
         return form
 
     @staticmethod
@@ -347,6 +313,9 @@ class BaseJob:
         vars = cls._get_vars()
         return_data = {}
 
+        if not isinstance(data, dict):
+            raise TypeError("Data should be a dictionary.")
+
         for field_name, value in data.items():
             # If a field isn't a var, skip it (e.g. `_commit`).
             try:
@@ -393,7 +362,7 @@ class BaseJob:
                 raise ValidationError({k: "Job data contained an unknown property"})
 
         # defer validation to the form object
-        f = self.as_form(data=data)
+        f = self.as_form(data=self.deserialize_data(data))
         if not f.is_valid():
             raise ValidationError(f.errors)
 
@@ -832,6 +801,10 @@ def get_jobs():
         # Iterate over all modules (Python files) found in any of the directory paths identified for the given grouping
         for importer, module_name, _ in pkgutil.iter_modules(path_list):
             try:
+                # Remove cached module to ensure consistency with filesystem
+                if module_name in sys.modules:
+                    del sys.modules[module_name]
+
                 # Dynamically import this module to make its contents (job(s)) available to Python
                 module = importer.find_module(module_name).load_module(module_name)
             except Exception as exc:
@@ -965,7 +938,14 @@ def run_job(data, request, job_result_pk, commit=True, *args, **kwargs):
     """
     from nautobot.extras.models import JobResult  # avoid circular import
 
-    job_result = JobResult.objects.get(pk=job_result_pk)
+    # Getting the correct job result can fail if the stored data cannot be serialized.
+    # Catching `TypeError: the JSON object must be str, bytes or bytearray, not int`
+    try:
+        job_result = JobResult.objects.get(pk=job_result_pk)
+    except TypeError as e:
+        logger.error(f"Unable to serialize data for job {job_result_pk}")
+        logger.error(e)
+        return False
 
     job_class = get_job(job_result.name)
     if not job_class:
@@ -979,26 +959,46 @@ def run_job(data, request, job_result_pk, commit=True, *args, **kwargs):
         job_result.completed = timezone.now()
         job_result.save()
         return False
-    job = job_class()
-    job.job_result = job_result
-
-    # Capture the file IDs for any FileProxy objects created so we can cleanup later.
-    file_fields = list(job._get_file_vars())
-    file_ids = [data[f] for f in file_fields]
-
-    # Attempt to resolve serialized data back into original form by creating querysets or model instances
-    # If we fail to find any objects, we consider this a job execution error, and fail.
-    # This might happen when a job sits on the queue for a while (i.e. scheduled) and data has changed
-    # or it might be bad input from an API request, or manual execution.
 
     try:
-        data = job_class.deserialize_data(data)
-    except ObjectDoesNotExist as e:
+        job = job_class()
+        job.active_test = "initialization"
+        job.job_result = job_result
+    except Exception as error:
+        logger.error("Error initializing job object.")
+        logger.error(error)
+        stacktrace = traceback.format_exc()
+        job_result.log_failure(f"Error initializing job:\n```\n{stacktrace}\n```")
         job_result.set_status(JobResultStatusChoices.STATUS_ERRORED)
-        job.log_failure(message=e)
         job_result.completed = timezone.now()
         job_result.save()
-        job.delete_files(*file_ids)  # Cleanup FileProxy objects
+        return False
+
+    try:
+        # Capture the file IDs for any FileProxy objects created so we can cleanup later.
+        file_ids = None
+        file_fields = list(job._get_file_vars())
+        file_ids = [data[f] for f in file_fields]
+
+        # Attempt to resolve serialized data back into original form by creating querysets or model instances
+        # If we fail to find any objects, we consider this a job execution error, and fail.
+        # This might happen when a job sits on the queue for a while (i.e. scheduled) and data has changed
+        # or it might be bad input from an API request, or manual execution.
+
+        data = job_class.deserialize_data(data)
+    except Exception:
+        job_result.set_status(JobResultStatusChoices.STATUS_ERRORED)
+        stacktrace = traceback.format_exc()
+        job_result.log(
+            f"Error initializing job:\n```\n{stacktrace}\n```",
+            level_choice=LogLevelChoices.LOG_FAILURE,
+            grouping="initialization",
+            logger=logger,
+        )
+        job_result.completed = timezone.now()
+        job_result.save()
+        if file_ids:
+            job.delete_files(*file_ids)  # Cleanup FileProxy objects
         return False
 
     if job.read_only:
@@ -1025,6 +1025,7 @@ def run_job(data, request, job_result_pk, commit=True, *args, **kwargs):
         If the job is marked as read_only == True, then commit is forced to False and no log messages will be
         emitted related to reverting database changes.
         """
+        started = timezone.now()
         job.results["output"] = ""
         try:
             with transaction.atomic():
@@ -1066,6 +1067,12 @@ def run_job(data, request, job_result_pk, commit=True, *args, **kwargs):
             job_result.save()
             job.delete_files(*file_ids)  # Cleanup FileProxy objects
 
+        # record data about this jobrun in the schedule
+        if job_result.schedule:
+            job_result.schedule.total_run_count += 1
+            job_result.schedule.last_run_at = started
+            job_result.schedule.save()
+
         # Perform any post-run tasks
         job.active_test = "post_run"
         output = job.post_run()
@@ -1084,3 +1091,23 @@ def run_job(data, request, job_result_pk, commit=True, *args, **kwargs):
             _run_job()
     else:
         _run_job()
+
+
+@nautobot_task
+def scheduled_job_handler(*args, **kwargs):
+    """
+    A thin wrapper around JobResult.enqueue_job() that allows for it to be called as an async task
+    for the purposes of enqueuing scheduled jobs at their recurring intervals. Thus, JobResult.enqueue_job()
+    is responsible for enqueuing the actual job for execution and this method is the task executed
+    by the scheduler to kick off the job execution on a recurring interval.
+    """
+    from nautobot.extras.models import JobResult  # avoid circular import
+
+    user_pk = kwargs.pop("user")
+    user = User.objects.get(pk=user_pk)
+    name = kwargs.pop("name")
+    scheduled_job_pk = kwargs.pop("scheduled_job_pk")
+    schedule = ScheduledJob.objects.get(pk=scheduled_job_pk)
+
+    job_content_type = ContentType.objects.get(app_label="extras", model="job")
+    JobResult.enqueue_job(run_job, name, job_content_type, user, schedule=schedule, **kwargs)
