@@ -12,12 +12,12 @@ from django.db import models
 from django.urls import reverse
 from django.utils.safestring import mark_safe
 
-from nautobot.extras.choices import *
+from nautobot.extras.choices import CustomFieldFilterLogicChoices, CustomFieldTypeChoices
 from nautobot.extras.models import ChangeLoggedModel
 from nautobot.extras.tasks import delete_custom_field_data, update_custom_field_choice_data
 from nautobot.extras.utils import FeatureQuery, extras_features
+from nautobot.core.fields import AutoSlugField
 from nautobot.core.models import BaseModel
-from nautobot.utilities.fields import JSONArrayField
 from nautobot.utilities.forms import (
     CSVChoiceField,
     CSVMultipleChoiceField,
@@ -28,6 +28,7 @@ from nautobot.utilities.forms import (
     add_blank_choice,
 )
 from nautobot.utilities.querysets import RestrictedQuerySet
+from nautobot.utilities.templatetags.helpers import render_markdown
 from nautobot.utilities.utils import render_jinja2
 from nautobot.utilities.validators import validate_regex
 
@@ -57,7 +58,7 @@ class ComputedField(BaseModel, ChangeLoggedModel):
         on_delete=models.CASCADE,
         limit_choices_to=FeatureQuery("custom_fields"),
     )
-    slug = models.SlugField(max_length=100, unique=True, help_text="Internal field name")
+    slug = AutoSlugField(populate_from="label", help_text="Internal field name")
     label = models.CharField(max_length=100, help_text="Name of the field as displayed to users")
     description = models.CharField(max_length=200, blank=True)
     template = models.TextField(max_length=500, help_text="Jinja2 template code for field value")
@@ -195,6 +196,7 @@ class CustomFieldManager(models.Manager.from_queryset(RestrictedQuerySet)):
         return self.get_queryset().filter(content_types=content_type)
 
 
+@extras_features("webhooks")
 class CustomField(BaseModel):
     content_types = models.ManyToManyField(
         to=ContentType,
@@ -207,30 +209,34 @@ class CustomField(BaseModel):
         max_length=50,
         choices=CustomFieldTypeChoices,
         default=CustomFieldTypeChoices.TYPE_TEXT,
+        help_text="The type of value(s) allowed for this field.",
     )
-    name = models.CharField(max_length=50, unique=True, help_text="Internal field name")
+    # TODO: Migrate custom field model from name to slug #464
+    name = models.CharField(max_length=50, unique=True, verbose_name="Slug", help_text="URL-friendly unique shorthand.")
     label = models.CharField(
         max_length=50,
         blank=True,
-        help_text="Name of the field as displayed to users (if not provided, " "the field's name will be used)",
+        help_text="Name of the field as displayed to users (if not provided, the field's slug will be used.)",
     )
-    description = models.CharField(max_length=200, blank=True)
+    description = models.CharField(max_length=200, blank=True, help_text="A helpful description for this field.")
     required = models.BooleanField(
         default=False,
-        help_text="If true, this field is required when creating new objects " "or editing an existing object.",
+        help_text="If true, this field is required when creating new objects or editing an existing object.",
     )
     filter_logic = models.CharField(
         max_length=50,
         choices=CustomFieldFilterLogicChoices,
         default=CustomFieldFilterLogicChoices.FILTER_LOOSE,
-        help_text="Loose matches any instance of a given string; exact " "matches the entire field.",
+        help_text="Loose matches any instance of a given string; Exact matches the entire field.",
     )
     default = models.JSONField(
         encoder=DjangoJSONEncoder,
         blank=True,
         null=True,
-        help_text="Default value for the field (must be a JSON value). Encapsulate "
-        'strings with double quotes (e.g. "Foo").',
+        help_text=(
+            "Default value for the field (must be a JSON value). Encapsulate strings with double quotes (e.g. "
+            '"Foo").'
+        ),
     )
     weight = models.PositiveSmallIntegerField(
         default=100, help_text="Fields with higher weights appear lower in a form."
@@ -239,13 +245,13 @@ class CustomField(BaseModel):
         blank=True,
         null=True,
         verbose_name="Minimum value",
-        help_text="Minimum allowed value (for numeric fields)",
+        help_text="Minimum allowed value (for numeric fields).",
     )
     validation_maximum = models.BigIntegerField(
         blank=True,
         null=True,
         verbose_name="Maximum value",
-        help_text="Maximum allowed value (for numeric fields)",
+        help_text="Maximum allowed value (for numeric fields).",
     )
     validation_regex = models.CharField(
         blank=True,
@@ -253,7 +259,8 @@ class CustomField(BaseModel):
         max_length=500,
         verbose_name="Validation regex",
         help_text="Regular expression to enforce on text field values. Use ^ and $ to force matching of entire string. "
-        "For example, <code>^[A-Z]{3}$</code> will limit values to exactly three uppercase letters.",
+        "For example, <code>^[A-Z]{3}$</code> will limit values to exactly three uppercase letters. Regular "
+        "expression on select and multi-select will be applied at <code>Custom Field Choices</code> definition.",
     )
 
     objects = CustomFieldManager()
@@ -264,6 +271,11 @@ class CustomField(BaseModel):
     def __str__(self):
         return self.label or self.name.replace("_", " ").capitalize()
 
+    # TODO: Migrate property to actual model attribute #464
+    @property
+    def slug(self):
+        return self.name
+
     def clean(self):
         super().clean()
 
@@ -294,10 +306,12 @@ class CustomField(BaseModel):
         regex_types = (
             CustomFieldTypeChoices.TYPE_TEXT,
             CustomFieldTypeChoices.TYPE_URL,
+            CustomFieldTypeChoices.TYPE_SELECT,
+            CustomFieldTypeChoices.TYPE_MULTISELECT,
         )
         if self.validation_regex and self.type not in regex_types:
             raise ValidationError(
-                {"validation_regex": "Regular expression validation is supported only for text and URL fields"}
+                {"validation_regex": "Regular expression validation is supported only for text, URL and select fields"}
             )
 
         # Choices can be set only on selection fields
@@ -395,8 +409,10 @@ class CustomField(BaseModel):
 
         field.model = self
         field.label = str(self)
+
         if self.description:
-            field.help_text = self.description
+            # Avoid script injection and similar attacks! Output HTML but only accept Markdown as input
+            field.help_text = render_markdown(self.description)
 
         return field
 
@@ -465,7 +481,14 @@ class CustomField(BaseModel):
 
         delete_custom_field_data.delay(self.name, content_types)
 
+    def get_absolute_url(self):
+        return reverse("extras:customfield", args=[self.name])
 
+
+@extras_features(
+    "graphql",
+    "webhooks",
+)
 class CustomFieldChoice(BaseModel):
     """
     The custom field choice is used to store the possible set of values for a selection type custom field
@@ -493,6 +516,9 @@ class CustomFieldChoice(BaseModel):
         if self.field.type not in (CustomFieldTypeChoices.TYPE_SELECT, CustomFieldTypeChoices.TYPE_MULTISELECT):
             raise ValidationError("Custom field choices can only be assigned to selection fields.")
 
+        if not re.match(self.field.validation_regex, self.value):
+            raise ValidationError(f"Value must match regex {self.field.validation_regex} got {self.value}.")
+
     def save(self, *args, **kwargs):
         """
         When a custom field choice is saved, perform logic that will update data across all custom field data.
