@@ -1,8 +1,5 @@
-import json
-
 from nautobot.dcim.models import DeviceRole
-from nautobot.extras.jobs import *
-
+from nautobot.extras.jobs import Job, ObjectVar, MultiObjectVar
 
 name = "Object Vars"
 
@@ -18,17 +15,13 @@ class TestObjectVars(Job):
         role = data["role"]
         roles = data["roles"]
 
-        # Log the data as JSON so we can pull it back out for testing.
-        self.log_info(
-            obj=json.dumps(
-                {
-                    "role": str(role.pk),
-                    "roles": [str(r) for r in roles.values_list("pk", flat=True)],
-                }
-            )
-        )
+        # store the object data for later
+        self.job_result.data["object_vars"] = {
+            "role": str(role.pk),
+            "roles": [str(r) for r in roles.values_list("pk", flat=True)],
+        }
 
-        self.log_warning(f"Role: {role}")
+        self.log_info(f"Role: {role}")
         self.log_warning(f"Roles: {roles}")
 
         self.log_success(message="Job didn't crash!")
