@@ -42,12 +42,20 @@ def clone_button(instance):
 
 
 @register.inclusion_tag("buttons/edit.html")
-def edit_button(instance, use_pk=False):
+def edit_button(instance, use_pk=False, key="slug"):
+    """
+    Render a button to edit a model instance.
+
+    Args:
+        instance: Model record.
+        use_pk: If True, use the primary key instead of any specified "key" field. (Deprecated, use `key="pk"` instead)
+        key: The attribute on the model to use for reverse URL lookup.
+    """
     viewname = _get_viewname(instance, "edit")
 
     # Assign kwargs
-    if hasattr(instance, "slug") and not use_pk:
-        kwargs = {"slug": instance.slug}
+    if hasattr(instance, key) and not use_pk:
+        kwargs = {key: getattr(instance, key)}
     else:
         kwargs = {"pk": instance.pk}
 
@@ -59,12 +67,20 @@ def edit_button(instance, use_pk=False):
 
 
 @register.inclusion_tag("buttons/delete.html")
-def delete_button(instance, use_pk=False):
+def delete_button(instance, use_pk=False, key="slug"):
+    """
+    Render a button to delete a model instance.
+
+    Args:
+        instance: Model record.
+        use_pk: If True, use the primary key instead of any specified "key" field. (Deprecated, use `key="pk"` instead)
+        key: The attribute on the model to use for reverse URL lookup.
+    """
     viewname = _get_viewname(instance, "delete")
 
     # Assign kwargs
-    if hasattr(instance, "slug") and not use_pk:
-        kwargs = {"slug": instance.slug}
+    if hasattr(instance, key) and not use_pk:
+        kwargs = {key: getattr(instance, key)}
     else:
         kwargs = {"pk": instance.pk}
 
